import java.awt.Color;
import java.awt.Graphics;
import java.net.InetAddress;

/**
 * @author bachmaer
 * @author opyrchal
 *         <p/>
 *         Super class for all moving objects in the space game. Unique identifies each
 *         Spacecraft using a ip address and a port number. Maintains position and
 *         heading state for SpaceCraft objects.
 */
public class SpaceCraft {

    // IP address used to uniquely identify the SpaceCraft
    InetAddress ID;
    int IDport;

    // Two dimensional position of the SpaceCraft
    int xPosition, yPosition;

    // Current heading of the SpaceCraft
    int heading = Constants.NORTH;


    /**
     * Accessor method for the heading.
     *
     * @return current heading
     */
    public int getHeading() {
        return heading;

    } // end getHeading


    /**
     * Checks to see if this SpaceCraft has the same ID and port number
     * as the input parameter.
     *
     * @param subject SpaceCraft being checked for equality
     * @return true if the SpaceCraft have same ID and port, false otherwise
     */
    public boolean equals(SpaceCraft subject) {
        System.out.println("EQUALS: my id: " + this.ID +
                " my port: " + this.IDport);
        System.out.println("     other id: " + subject.ID +
                " other port: " + subject.IDport);
        return (this.ID.equals(subject.ID) && (this.IDport == subject.IDport));

    } // end equals


    /**
     * Accessor method for the xPosition.
     *
     * @return current x position
     */
    public int getXPosition() {
        return xPosition;

    } // end getXPosition


    /**
     * Accessor method for the yPosition.
     *
     * @return current y position
     */
    public int getYPosition() {
        return yPosition;

    } // end getYPosition


    /**
     * Mutator method for the heading. If input parameter contains an illegal
     * heading it leaves the heading unchanged.
     *
     * @param heading parmeter set to 1, 2, 3, or 4
     */
    public void setHeading(int heading) {
        this.heading =
                (heading > 0 && heading <= Constants.WEST) ? heading : this.heading;

    } // end setHeading


    /**
     * Mutator method for the x positon. If input parameter contains an illegal
     * position it leaves the position unchanged.
     *
     * @param x new x position
     */
    public void setX(int x) {
        this.xPosition =
                (x > 0 && x < Constants.MAX_SECTOR_X) ? x : this.xPosition;

    } // end setX


    /**
     * Mutator method for the y positon. If input parameter contains an illegal
     * position it leaves the position unchanged.
     *
     * @param y new y position
     */
    public void setY(int y) {
        this.yPosition =
                (y > 0 && y < Constants.MAX_SECTOR_Y) ? y : this.yPosition;

    } // end setY

    /**
     * Default rendering method. Renders the SpaceCraft as a yellow circle.
     *
     * @return current x position
     */
    public void draw(Graphics g) {
        // Draw the spacecraft with the appropriate heading
        g.setColor(Color.green);
        g.fillOval(xPosition - Constants.OBJECT_WIDTH / 2,
                yPosition - Constants.OBJECT_WIDTH / 2,
                Constants.OBJECT_WIDTH, Constants.OBJECT_WIDTH);

    } // end draw


    /**
     * Constructs a string representation of the the craft.
     *
     * @return string representation of the SpaceCraft
     */
    public String toString() {

        return "Address: " + ID + " heading: "
                + heading + " x: " + xPosition + " y: " + yPosition;

    } // end toString


} // end SpaceCraft
